/* Generated by wayland-scanner 1.21.0 */

/*
 * Copyright © 2015, 2016 Red Hat
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface gtk_primary_selection_device_interface;
extern const struct wl_interface gtk_primary_selection_offer_interface;
extern const struct wl_interface gtk_primary_selection_source_interface;
extern const struct wl_interface wl_seat_interface;

static const struct wl_interface *gtk_primary_selection_types[] = {
	NULL,
	NULL,
	&gtk_primary_selection_source_interface,
	&gtk_primary_selection_device_interface,
	&wl_seat_interface,
	&gtk_primary_selection_source_interface,
	NULL,
	&gtk_primary_selection_offer_interface,
	&gtk_primary_selection_offer_interface,
};

static const struct wl_message gtk_primary_selection_device_manager_requests[] = {
	{ "create_source", "n", gtk_primary_selection_types + 2 },
	{ "get_device", "no", gtk_primary_selection_types + 3 },
	{ "destroy", "", gtk_primary_selection_types + 0 },
};

WL_PRIVATE const struct wl_interface gtk_primary_selection_device_manager_interface = {
	"gtk_primary_selection_device_manager", 1,
	3, gtk_primary_selection_device_manager_requests,
	0, NULL,
};

static const struct wl_message gtk_primary_selection_device_requests[] = {
	{ "set_selection", "?ou", gtk_primary_selection_types + 5 },
	{ "destroy", "", gtk_primary_selection_types + 0 },
};

static const struct wl_message gtk_primary_selection_device_events[] = {
	{ "data_offer", "n", gtk_primary_selection_types + 7 },
	{ "selection", "?o", gtk_primary_selection_types + 8 },
};

WL_PRIVATE const struct wl_interface gtk_primary_selection_device_interface = {
	"gtk_primary_selection_device", 1,
	2, gtk_primary_selection_device_requests,
	2, gtk_primary_selection_device_events,
};

static const struct wl_message gtk_primary_selection_offer_requests[] = {
	{ "receive", "sh", gtk_primary_selection_types + 0 },
	{ "destroy", "", gtk_primary_selection_types + 0 },
};

static const struct wl_message gtk_primary_selection_offer_events[] = {
	{ "offer", "s", gtk_primary_selection_types + 0 },
};

WL_PRIVATE const struct wl_interface gtk_primary_selection_offer_interface = {
	"gtk_primary_selection_offer", 1,
	2, gtk_primary_selection_offer_requests,
	1, gtk_primary_selection_offer_events,
};

static const struct wl_message gtk_primary_selection_source_requests[] = {
	{ "offer", "s", gtk_primary_selection_types + 0 },
	{ "destroy", "", gtk_primary_selection_types + 0 },
};

static const struct wl_message gtk_primary_selection_source_events[] = {
	{ "send", "sh", gtk_primary_selection_types + 0 },
	{ "cancelled", "", gtk_primary_selection_types + 0 },
};

WL_PRIVATE const struct wl_interface gtk_primary_selection_source_interface = {
	"gtk_primary_selection_source", 1,
	2, gtk_primary_selection_source_requests,
	2, gtk_primary_selection_source_events,
};

